"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = __importDefault(require("ramda"));
var node_ops_1 = require("./node-ops");
var utils_1 = require("./utils");
var findActionCreators = function (ast, actionCreatorsArg) {
    if (actionCreatorsArg.type === "Identifier") {
        var name = ramda_1.default.propOr(undefined, 'name', actionCreatorsArg);
        if (name === 'undefined') {
            return [];
        }
        else {
            var decl = node_ops_1.searchByIdName(ast, name);
            var props = ramda_1.default.pathOr([], ['init', 'properties'], decl);
            return utils_1.noNulls(props.map(ramda_1.default.path(['key', 'name'])));
        }
    }
};
var findMSTPReturnArgs = function (ast, mapStateArg) {
    if (mapStateArg.type === "Identifier") {
        var name = ramda_1.default.propOr(undefined, 'name', mapStateArg);
        if (name === 'undefined') {
            return [];
        }
        else {
            var decl = node_ops_1.searchByIdName(ast, name);
            var returnStmt = node_ops_1.searchByType(decl, 'ReturnStatement');
            var props = ramda_1.default.pathOr([], ['argument', 'properties'], returnStmt);
            return utils_1.noNulls(props.map(ramda_1.default.path(['key', 'name'])));
        }
    }
};
exports.getReduxDetails = function (ast) {
    var connectCall = node_ops_1.searchBy(ast, function (node) { return node_ops_1.byType('CallExpression') && ramda_1.default.path(['callee', 'callee', 'name'], node) === 'connect'; });
    var funcNames = [];
    var returnedPropNames = [];
    if (connectCall === undefined) {
        return {
            connected: false,
            funcNames: funcNames,
            returnedPropNames: returnedPropNames,
            subject: '',
        };
    }
    else {
        var subject = ramda_1.default.propOr('unknown', 'name', connectCall.arguments[0]);
        var calleeArgs = ramda_1.default.pathOr([], ['callee', 'arguments'], connectCall);
        var mapStateArg = calleeArgs[0];
        var actionCreatorsArg = calleeArgs[1];
        funcNames = findActionCreators(ast, actionCreatorsArg);
        returnedPropNames = findMSTPReturnArgs(ast, mapStateArg);
        return {
            connected: true,
            funcNames: funcNames,
            returnedPropNames: returnedPropNames,
            subject: subject,
        };
    }
};
//# sourceMappingURL=redux.js.map