"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var es2015_1 = __importDefault(require("estel-estree-builder/generated/es2015"));
var ramda_1 = __importDefault(require("ramda"));
var acorn_1 = require("./acorn");
var node_ops_1 = require("./node-ops");
var tree_builders_1 = require("./tree-builders");
var utils_1 = require("./utils");
var _a = require('./js-gen'), genJs = _a.genJs, genJsList = _a.genJsList;
var varDecName = function (vd) {
    return vd && vd.id && vd.id.name;
};
exports.letForPropTypes = function (propDefs) {
    if (propDefs && propDefs.map) {
        return propDefs.map(function (p) { return p.mockVar; });
    }
};
exports.jestMockFn = tree_builders_1.callFn(tree_builders_1.dot(es2015_1.default.identifier('jest'), 'fn'), []);
var stringMock = function (n) { return es2015_1.default.literal(n + 'MockValue'); };
exports.propTypeToMock = {
    func: function () { return exports.jestMockFn; },
    number: function () { return es2015_1.default.literal(1); },
    object: function () { return es2015_1.default.objectExpression([]); },
};
var propTypeMockName = function (name, type) {
    if (type === 'func') {
        return name + 'Mock';
    }
    else {
        return name + 'MockData';
    }
};
exports.processPropTypes = function (propDefs) {
    if (propDefs && propDefs.map) {
        return propDefs.map(function (p) {
            var propName = p.key.value || p.key.name;
            var propTypeName = p.value.property.name;
            var propTypeType = propTypeName === 'isRequired' ?
                p.value.object.property.name :
                propTypeName;
            var mockName = propTypeMockName(propName, propTypeType);
            var mockGen = exports.propTypeToMock[propTypeType] || stringMock;
            var mockVal = mockGen(propName);
            return {
                mockName: mockName,
                mockVal: mockVal,
                mockVar: tree_builders_1.lt(mockName),
                propName: propName,
            };
        });
    }
    else {
        return [];
    }
};
var buildSuiteForClassBased = function (classComponentNode, builderOptions, allNodes) {
    // allNodes.map(n => printNode(n))
    var className = varDecName(classComponentNode);
    var generateResult = tree_builders_1.cnst('result', tree_builders_1.callFn('renderComponent', []));
    var expectResult = tree_builders_1.callFn('expect', [es2015_1.default.identifier('result')]);
    var checkResult = tree_builders_1.callFn(tree_builders_1.dot(tree_builders_1.dot(tree_builders_1.dot(expectResult, 'to'), 'deep'), 'equal'), [es2015_1.default.identifier('result')]);
    var propTypes = node_ops_1.searchForPropTypes(allNodes, className);
    var propTypeOutputs = exports.processPropTypes(propTypes);
    var beforeEach = tree_builders_1.buildBeforeEach(propTypeOutputs);
    var suite = tree_builders_1.callFn(es2015_1.default.identifier('describe'), [
        es2015_1.default.literal("render " + className),
        tree_builders_1.arrow.apply(void 0, exports.letForPropTypes(propTypeOutputs).concat([beforeEach,
            tree_builders_1.buildRenderFunc(className, propTypeOutputs),
            tree_builders_1.buildItBlock('can render', [generateResult, checkResult])])),
    ]);
    return {
        defaultDepImports: {
            react: 'React',
        },
        namedDepImports: {
            chai: ['expect'],
            enzyme: ['shallow'],
        },
        namedSut: className,
        suite: suite,
    };
};
var buildSuiteForJsx = function (exportNode, _a) {
    var exportName = _a.exportName;
    var generateResult = tree_builders_1.cnst('result', tree_builders_1.callFn('renderComponent', [es2015_1.default.identifier(exportName)]));
    var expectResult = tree_builders_1.callFn('expect', [es2015_1.default.identifier('result')]);
    var checkResult = tree_builders_1.callFn(tree_builders_1.dot(tree_builders_1.dot(expectResult, 'to'), 'deepEqual'), [es2015_1.default.identifier('result')]);
    var suite = tree_builders_1.callFn('describe', [
        es2015_1.default.literal("render " + exportName),
        tree_builders_1.arrow(tree_builders_1.buildRenderFunc(exportName), tree_builders_1.buildItBlock('can render', [generateResult, checkResult])),
    ]);
    return {
        namedSut: exportName,
        suite: suite,
    };
};
var buildSuiteForMapStateToProps = function (exportNode, _a) {
    var exportName = _a.exportName;
    var generateResult = tree_builders_1.cnst('result', tree_builders_1.callFn(exportName, [es2015_1.default.identifier('state')]));
    var suite = tree_builders_1.callFn('describe', [
        es2015_1.default.literal(exportName),
        tree_builders_1.arrow(tree_builders_1.buildItBlock('can render', [generateResult])),
    ]);
    return {
        namedSut: exportName,
        suite: suite,
    };
};
var buildSuiteForBasicFunc = function (exportNode, _a) {
    var exportName = _a.exportName;
    var generateResult = tree_builders_1.cnst('result', tree_builders_1.callFn(exportName, []));
    var expectResult = tree_builders_1.callFn('expect', [es2015_1.default.identifier('result')]);
    var checkResult = tree_builders_1.callFn(tree_builders_1.dot(tree_builders_1.dot(expectResult, 'to'), 'deepEqual'), [es2015_1.default.objectExpression([])]);
    var suite = tree_builders_1.callFn('describe', [
        es2015_1.default.literal(exportName),
        tree_builders_1.arrow(tree_builders_1.buildItBlock('works', [generateResult, checkResult])),
    ]);
    return {
        namedSut: exportName,
        suite: suite,
    };
};
var addSut = function (builderResult, _a) {
    var isDefaultExport = _a.isDefaultExport, exportName = _a.exportName, inputModulePath = _a.inputModulePath;
    var sutName = exportName || builderResult.namedSut;
    var namedSut = isDefaultExport ? null : sutName;
    var defaultSut = isDefaultExport ? sutName : null;
    return __assign({}, builderResult, { defaultSut: defaultSut,
        inputModulePath: inputModulePath,
        namedSut: namedSut });
};
// REVISIT: The exportNode should be the content of exportNode.declaration, to iterate multi export statements
// Right now it only catches the first one
var buildSuiteFor = function (exportNode, allNodes, inputModulePath) {
    var exportDecl = exportNode.declaration;
    var isTransitiveExport = exportDecl.type === 'Identifier';
    var declaringNode = isTransitiveExport ?
        node_ops_1.searchByIdName(allNodes, exportDecl.name) :
        exportNode;
    // Bail out if the export is for something wrapped in a call to connect
    // This implies its a connected component and it can't be tested
    // This *could* be applied to anything that is a wrapped export
    if (exportDecl.callee &&
        exportDecl.callee.callee &&
        exportDecl.callee.callee.name === 'connect') {
        return;
    }
    // printNode(exportNode, 0)
    var varDec = node_ops_1.searchByType(declaringNode, "VariableDeclarator");
    var classComponentNode = node_ops_1.searchBySuperClass(declaringNode, "Component");
    var exportName = varDecName(classComponentNode || varDec);
    var hasJsx = node_ops_1.searchByType(declaringNode, "JSXElement");
    var arrowFunc = node_ops_1.searchByType(declaringNode, "ArrowFunctionExpression");
    var builderOptions = {
        exportName: exportName,
        inputModulePath: inputModulePath,
        isDefaultExport: exportNode.type === "ExportDefaultDeclaration",
    };
    var builderResult;
    if (exportName === 'mapStateToProps') {
        builderResult = buildSuiteForMapStateToProps(exportNode, builderOptions);
    }
    else if (classComponentNode) {
        builderResult = buildSuiteForClassBased(classComponentNode, builderOptions, allNodes);
    }
    else if (hasJsx) {
        builderResult = buildSuiteForJsx(exportNode, builderOptions);
    }
    else if (arrowFunc) {
        builderResult = buildSuiteForBasicFunc(exportNode, builderOptions);
    }
    else {
        return;
    }
    return addSut(builderResult, builderOptions);
};
var buildSuitesFor = function (exportingNodes, allNodes, inputModulePath) {
    return utils_1.noNulls(exportingNodes.map(function (en) { return buildSuiteFor(en, allNodes, inputModulePath); }));
};
var bySuiteName = function (nodes) {
    return ramda_1.default.indexBy(function (n) { return n.arguments[0].value; }, nodes.map(function (n) { return n.suite; }));
};
var mergeSuites = function (existing, incoming) {
    var existingByName = bySuiteName(existing);
    var incomingByName = bySuiteName(incoming);
    return ramda_1.default.values(ramda_1.default.merge(existingByName, incomingByName));
};
exports.generateTests = function (inputText, existingText, inputModulePath) {
    if (!inputText) {
        return '';
    }
    var inputAST = acorn_1.parse(inputText);
    var inputNodes = inputAST.body;
    // const namedExportNames = namedExports.map(findExportName)
    // Grab the top level export statements
    var namedExportNodes = inputNodes.filter(node_ops_1.byType("ExportNamedDeclaration"));
    var defaultExportNode = inputNodes.find(node_ops_1.byType("ExportDefaultDeclaration"));
    var exportNodes = defaultExportNode ? namedExportNodes.concat(defaultExportNode) : namedExportNodes;
    var exportSuiteTrees = buildSuitesFor(exportNodes, inputNodes, inputModulePath);
    var outputAST = acorn_1.parse(existingText);
    var outputNodes = outputAST.body;
    var existingTopLevelSuites = node_ops_1.filterByType(outputNodes, "CallExpression")
        .filter(ramda_1.default.pathEq(['callee', 'name'], 'describe'))
        .map(function (exp) {
        var arg0 = exp.arguments[0];
        return {
            name: ramda_1.default.propOr('mystery describe block', 'value', arg0),
            suite: exp,
        };
    });
    var mergedSuites = mergeSuites(existingTopLevelSuites, exportSuiteTrees);
    var namedImportMaps = utils_1.noNulls(exportSuiteTrees
        .map(function (t) { return t.namedDepImports; }));
    var defaultImportMaps = utils_1.noNulls(exportSuiteTrees
        .map(function (t) { return t.defaultDepImports; }));
    var importStmts = tree_builders_1.buildImportStmts(namedImportMaps, defaultImportMaps, exportSuiteTrees);
    var namedExportSuites = mergedSuites.map(genJs).join('\n');
    var content = "\n" + genJsList(importStmts) + "\n\n" + namedExportSuites;
    return content;
};
//# sourceMappingURL=test-content.js.map