"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require('path');
var _a = require('../path-helpers'), removeExtension = _a.removeExtension, trimLeadingFolders = _a.trimLeadingFolders, attachLeadingFolders = _a.attachLeadingFolders;
exports.parallelDirs = function (projectFuncs, projectPaths, settings) {
    var packagePath = settings.packagePath, sourcePath = settings.sourcePath, testPath = settings.testPath, testSuffix = settings.testSuffix;
    var testInsideProject = projectPaths.testInsideProject;
    var flipSrcPathToTestPath = function (sourceFileWithinProject) {
        var sourceFileMinusExt = removeExtension(projectFuncs.sourcePathWithinSrc(sourceFileWithinProject));
        var pathPieces = [testInsideProject, sourceFileMinusExt, testSuffix, '.js'];
        return path.normalize(pathPieces.join(''));
    };
    return {
        sourceFileWPToTestFileWP: function (sourceFileWithinProject) {
            return flipSrcPathToTestPath(sourceFileWithinProject);
        },
        isPathWPTestFile: function (p) {
            var trimmedPathResult = trimLeadingFolders(p, [packagePath, testPath]);
            if (!trimmedPathResult.success) {
                return false;
            }
            return removeExtension(trimmedPathResult.value).endsWith(testSuffix);
        },
        testFileWPToSourceFileWP: function (p) {
            var trimmedPathResult = trimLeadingFolders(p, [packagePath, testPath]);
            var baseFile = removeExtension(trimmedPathResult.value).slice(0, -testSuffix.length);
            var baseFilePlusPrefix = attachLeadingFolders(baseFile, [packagePath, sourcePath]);
            return baseFilePlusPrefix + ".js";
        },
    };
};
//# sourceMappingURL=parallel-dirs.js.map